//#############################################################################   
//
//	Unterprogramme usb4all.c
//
//#############################################################################

#include "usb4all.h"

//#############################################################################
//
//	Unterprogram   usb4all_initialize
//
//#############################################################################

int usb4all_initialize(struct usb4all_t *usb4con)
{
  #ifdef DEBUG
  fprintf(stdout, "function usb4all_initialize\n");
  #endif		    
  usb_init();
  memset(usb4con,0,sizeof(*usb4con));
  return 0;
}

//#############################################################################
//
//	Unterprogram   usb4all_connect
//
//#############################################################################
   
int usb4all_connect(struct usb4all_t *usb4con)
{
  struct usb_bus *bus;
  struct usb_device *dev;
  struct usb_interface_descriptor *iface;
  usb_dev_handle *udev;
  int mode, usbmode, ep_in, ep_out;
  
  #ifdef DEBUG
  fprintf(stdout, "function usb4all_connect");
  #endif		    

  usb_find_busses();
  usb_find_devices();

  for (bus = usb_busses; bus; bus = bus->next) {
      for (dev = bus->devices; dev; dev = dev->next) {
 	  udev = usb_open(dev);
	  if (udev) {
              #ifdef DEBUG
             fprintf(stdout, "\nScan VID:%04X PID:%04X",dev->descriptor.idVendor,dev->descriptor.idProduct);
              #endif
	     if (dev->descriptor.idVendor ==  VENDOR &&
	         dev->descriptor.idProduct == PRODUCT &&
	         dev->descriptor.bNumConfigurations == 1 &&
		 dev->config[0].bNumInterfaces == 1 &&
		 dev->config[0].interface[0].num_altsetting == 1 &&
		 dev->config[0].interface[0].altsetting[0].bNumEndpoints==2) {
		 #ifdef DEBUG
		 fprintf(stdout, " --> found\n");
		 #endif		    
		iface = &dev->config[0].interface[0].altsetting[0];
		mode = MODE_NORMAL;
		usbmode = 0;
		ep_in = 0xFF;
		ep_out = 0xFF;
		// feststellen ob bul oder interrupt benutzt vwerden muss
		if ((iface->endpoint[0].bmAttributes&USB_ENDPOINT_TYPE_MASK)
		    == USB_ENDPOINT_TYPE_INTERRUPT &&
		    (iface->endpoint[1].bmAttributes&USB_ENDPOINT_TYPE_MASK)
		    == USB_ENDPOINT_TYPE_INTERRUPT) {
		    // normal mode 
		    usbmode = MODE_INT;
                    #ifdef DEBUG
		    fprintf(stdout, "Seems to be in INT mode\n");
                    #endif		    
		}
		if ((iface->endpoint[0].bmAttributes&USB_ENDPOINT_TYPE_MASK) 
		    == USB_ENDPOINT_TYPE_BULK &&
		    (iface->endpoint[1].bmAttributes&USB_ENDPOINT_TYPE_MASK) 
		    == USB_ENDPOINT_TYPE_BULK) {
		    // boot loader mode 
		    usbmode = MODE_BULK;
                    #ifdef DEBUG
		    fprintf(stdout, "Seems to be in BULK mode\n");
                    #endif		    
                }
		if (usbmode == MODE_INT || usbmode == MODE_BULK) {
		   if (iface->endpoint[0].bEndpointAddress & USB_ENDPOINT_DIR_MASK) {
                      ep_in = iface->endpoint[0].bEndpointAddress;
                      } else {
		      ep_out = iface->endpoint[0].bEndpointAddress;
                }
		if (iface->endpoint[1].bEndpointAddress & USB_ENDPOINT_DIR_MASK) {
        	   ep_in = iface->endpoint[1].bEndpointAddress;
		   } else {
		   ep_out = iface->endpoint[1].bEndpointAddress;
		}
                if (ep_in != 0xFF && ep_out != 0xFF) {
		   usb4con->dev = udev;
		   usb4con->ep_in = ep_in & USB_ENDPOINT_ADDRESS_MASK ;
		   usb4con->ep_out = ep_out & USB_ENDPOINT_ADDRESS_MASK ;
		   usb4con->interface = iface->bInterfaceNumber;
		   usb4con->usbmode = usbmode;
		   /* try to get fw version and programmer type */
		   if (!usb_claim_interface(udev, usb4con->interface)) {
                      #ifdef DEBUG
                      fprintf(stdout, "Interface claimed\n");
                      #endif		    
		      usb_get_string_simple(usb4con->dev,
                                            dev->descriptor.iProduct,
					    usb4con->usb_name, sizeof(usb4con->usb_name));
                      return 0;
		   } else {
		      perror("Problem claiming interface");
                      if (errno == EPERM) fprintf(stderr, "You don't have sufficient rights (try sudo)\n");
 		      return -1;
                   }
                }
             }
	  }
          usb_close(udev);
          }
      }
  }
  #ifdef DEBUG
  fprintf(stdout, "\nDevice not found\n");
  #endif		    
  return -ENODEV;
}

//#############################################################################   
//
//	Unterprogram   usb4all_finalize
//
//#############################################################################
  
int usb4all_finalize(struct usb4all_t *usb4con)
{
  #ifdef DEBUG
  fprintf(stdout, "function usb4all_finalize\n");
  #endif		    
  if (usb4con->dev) {
      usb_release_interface(usb4con->dev,usb4con->interface);
      usb_reset(usb4con->dev);
      usb_close(usb4con->dev);
  }
  return 0;
}

//#############################################################################   
//
//	Unterprogram   usb4all_data_io
//
//#############################################################################

int usb4all_data_io(struct usb4all_t *usb4con, unsigned char *data_in, int data_in_cnt, unsigned char *data_out, int data_out_cnt)
{
	int rc;
	
#ifdef DEBUG
	int i;	
	fprintf(stdout, "=>");
	for (i=0; i<data_in_cnt; i++) fprintf(stdout, " %02X", data_in[i]);
	fprintf(stdout, "\n");
#endif

	if (usb4con->usbmode == MODE_INT) {
		rc = usb_interrupt_write(usb4con->dev, usb4con->ep_out, (char*)data_in, data_in_cnt, TIMEOUT_WRITE);
	} else {
		rc = usb_bulk_write(usb4con->dev, usb4con->ep_out, (char*)data_in, data_in_cnt, TIMEOUT_WRITE);
	}

	if (rc < 0)		// Fehler
		return rc;
	if ((data_in[0] == CMD_RESET) || (data_in[0] == BOOT_RESET)) 
		return 0;
	if (rc != data_in_cnt) 
		return -EFAULT;

	if (usb4con->usbmode == MODE_INT) {
           rc = usb_interrupt_read(usb4con->dev, usb4con->ep_in,(char*)data_out,data_out_cnt,TIMEOUT_READ);
	} else {
	   rc = usb_bulk_read(usb4con->dev, usb4con->ep_in,(char*)data_out,data_out_cnt,TIMEOUT_READ);
	}

#ifdef DEBUG
	fprintf(stdout, "<=");
	if (rc>0) {
	   for (i=0; i<rc; i++) fprintf(stdout, " %02X", data_out[i]);
	}
	fprintf(stdout, "\n");
#endif

	return rc;		
}

//#############################################################################   
